package com.hai.service.impl;


import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.utils.CoordCommonUtil;
import com.hai.common.utils.MD5Util;
import com.hai.dao.HighMerchantStoreMapper;
import com.hai.entity.HighMerchantStore;
import com.hai.entity.HighMerchantStoreExample;
import com.hai.entity.SecUser;
import com.hai.model.HighMerchantStoreModel;
import com.hai.service.HighMerchantStoreService;
import com.hai.service.SecUserService;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


@Service("highMerchantStoreService")
public class HighMerchantStoreServiceImpl implements HighMerchantStoreService {

    @Resource
    private HighMerchantStoreMapper highMerchantStoreMapper;

    @Resource
    private SecUserService secUserService;

    @Override
    @Transactional(propagation= Propagation.REQUIRES_NEW)
    public void insertMerchantStore(HighMerchantStoreModel highMerchantStore) throws Exception {
        highMerchantStoreMapper.insert(highMerchantStore);

        if (highMerchantStore.getSecUser() == null) {
            highMerchantStore.setSecUser(new SecUser());
        }
        highMerchantStore.getSecUser().setUserName(highMerchantStore.getStoreName());
        if (StringUtils.isNotBlank(highMerchantStore.getSecUser().getTelephone())) {
            highMerchantStore.getSecUser().setLoginName(highMerchantStore.getSecUser().getTelephone());
            highMerchantStore.getSecUser().setPassword(MD5Util.encode(highMerchantStore.getSecUser().getPassword().getBytes()));
        } else {
            highMerchantStore.getSecUser().setLoginName(highMerchantStore.getStoreKey());
            highMerchantStore.getSecUser().setPassword(MD5Util.encode("123456".getBytes()));
        }
        highMerchantStore.getSecUser().setAdminFlag(1);
        highMerchantStore.getSecUser().setStatus(1);
        highMerchantStore.getSecUser().setRoleId(4L);
        highMerchantStore.getSecUser().setObjectType(3);
        highMerchantStore.getSecUser().setObjectId(highMerchantStore.getId());
        highMerchantStore.getSecUser().setCreateTime(new Date());
        highMerchantStore.getSecUser().setUpdateTime(new Date());
        secUserService.addUser(highMerchantStore.getSecUser());
    }

    @Override
    @Transactional(propagation= Propagation.REQUIRES_NEW)
    public void insertMerchantStoreList(List<HighMerchantStoreModel> highMerchantStoreList) throws Exception {
        for (HighMerchantStoreModel highMerchantStore : highMerchantStoreList) {
            insertMerchantStore(highMerchantStore);
        }
    }

    @Override
    @Transactional
    public void updateMerchantStore(HighMerchantStoreModel highMerchantStore) throws Exception {
        highMerchantStoreMapper.updateByPrimaryKey(highMerchantStore);

        // 查询主账号
        SecUser mainAccount = secUserService.getMainAccount(3, highMerchantStore.getId());
        if (mainAccount != null) {
            String userPass = MD5Util.encode(highMerchantStore.getSecUser().getPassword().getBytes());
            if (!mainAccount.getPassword().equals(userPass)) {
                mainAccount.setPassword(userPass);
            }
            mainAccount.setTelephone(highMerchantStore.getTelephone());
            secUserService.updateUser(mainAccount);
        }
    }

    @Override
    public void updateMerchantStoreDetail(HighMerchantStore highMerchantStore) throws Exception {
        highMerchantStoreMapper.updateByPrimaryKey(highMerchantStore);
    }

    @Override
    @Transactional
    public void deleteMerchantStore(Long id) {
        // 查询门店
        HighMerchantStore store = highMerchantStoreMapper.selectByPrimaryKey(id);
        if (store == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.MERCHANT_STORE_NOF_FOUND, "");
        }
        store.setStatus(0);
        store.setUpdateTime(new Date());
        highMerchantStoreMapper.updateByPrimaryKey(store);

        // 查询门店下的所有用户
        List<SecUser> userList = secUserService.getMerchantStoreUser(store.getId());
        for (SecUser user : userList) {
            user.setStatus(0);  // 删除
            secUserService.updateUser(user);
        }
    }

    @Override
    public HighMerchantStoreModel getMerchantStoreById(Long id) {
        HighMerchantStore merchantStore = highMerchantStoreMapper.selectByPrimaryKey(id);
        if (merchantStore == null) {
            return null;
        }
        HighMerchantStoreModel model = new HighMerchantStoreModel();
        BeanUtils.copyProperties(merchantStore, model);

        // 查询主账号
        SecUser mainAccount = secUserService.getMainAccount(3, merchantStore.getId());
        if (mainAccount != null) {
            model.setSecUser(mainAccount);
        }
        return model;
    }

    @Override
    public HighMerchantStoreModel getMerchantStoreByKey(String storeKey) {
        HighMerchantStoreExample example = new HighMerchantStoreExample();
        example.createCriteria().andStoreKeyEqualTo(storeKey);
        List<HighMerchantStore> list = highMerchantStoreMapper.selectByExample(example);
        if (list.size() > 0) {
            HighMerchantStore merchantStore = list.get(0);
            if (merchantStore == null) {
                return null;
            }
            HighMerchantStoreModel model = new HighMerchantStoreModel();
            BeanUtils.copyProperties(merchantStore, model);

            // 查询主账号
            SecUser mainAccount = secUserService.getMainAccount(3, merchantStore.getId());
            if (mainAccount != null) {
                model.setSecUser(mainAccount);
            }
            return model;
        }
        return null;
    }

    @Override
    public List<HighMerchantStore> getMerchantStoreList(Map<String, Object> map) {
        HighMerchantStoreExample example = new HighMerchantStoreExample();
        HighMerchantStoreExample.Criteria criteria = example.createCriteria();

        if (MapUtils.getLong(map, "companyId") != null) {
            criteria.andCompanyIdEqualTo(MapUtils.getLong(map, "companyId"));
        }

        if (MapUtils.getLong(map, "merchantId") != null) {
            criteria.andMerchantIdEqualTo(MapUtils.getLong(map, "merchantId"));
        }

        if (StringUtils.isNotBlank(MapUtils.getString(map, "storeKey"))) {
            criteria.andStoreKeyEqualTo(MapUtils.getString(map, "storeKey"));
        }

        if (StringUtils.isNotBlank(MapUtils.getString(map, "storeName"))) {
            criteria.andStoreNameLike("%" + MapUtils.getString(map, "storeName") + "%");
        }

        if (StringUtils.isNotBlank(MapUtils.getString(map, "telephone"))) {
            criteria.andTelephoneEqualTo(MapUtils.getString(map, "telephone"));
        }

        criteria.andStatusEqualTo(1);

        example.setOrderByClause("create_time desc");
        return highMerchantStoreMapper.selectByExample(example);
    }

    @Override
    public List<HighMerchantStore> getStoreListByLongitude(Map<String, Object> map) {
        List<HighMerchantStore> storeList = getMerchantStoreList(map);
        for (HighMerchantStore store : storeList) {
            double distance = CoordCommonUtil.getDistance(Double.valueOf(store.getLatitude()), Double.valueOf(store.getLongitude()), Double.valueOf(map.get("latitude").toString()), Double.valueOf(map.get("longitude").toString()));
            store.setDistance(Math.round(distance/100d)/10d);
        }
        List<HighMerchantStore> collect = storeList.stream().sorted(Comparator.comparing(HighMerchantStore::getDistance)).collect(Collectors.toList());
        return collect;
    }


}
